@@ -13,6 +13,7 @@ from dslibrary import DSLibrary
 from dslibrary.front import EVALUATION_RESULT_ALIAS, METRICS_ALIAS, DSLibraryException, DSLibraryDataFormatException
 from dslibrary.metadata import Metadata
 from dslibrary.transport.to_local import DSLibraryLocal
+from dslibrary.utils.file_utils import write_stream_with_read_on_close
 
 
 class TestFront(unittest.TestCase):
@@ -669,3 +670,26 @@ class TestFront(unittest.TestCase):
         MyInst().write_resource("x.csv", dask.dataframe.from_pandas(df, npartitions=2), dask=False)
         self.assertEqual(files["x.csv"].getvalue(), b"x\n0\n1\n2\n3\n4\n5\n6\n")
 
+    def test_read_write_resource__to_filesystem(self):
+        """
+        read_resource() and write_resource() accept a 'filesystem' argument which points them to a particular
+        custom filesystem.
+        """
+        files = {}
+        class MyInst(DSLibrary):
+            def get_filesystem_connection(self, resource_name: str, for_write: bool=False, **kwargs):
+                assert resource_name == "my-s3"
+                assert kwargs == {}
+                class MyFS(object):
+                    def open(self, path, mode='rb'):
+                        if 'r' in mode:
+                            return io.BytesIO(files[path])
+                        assert for_write
+                        def written(fh):
+                            files[path] = fh.read()
+                        return write_stream_with_read_on_close(mode, r_mode='rb', on_close=written)
+                return MyFS()
+        MyInst().write_resource('new.txt', b'hello123', filesystem='my-s3')
+        self.assertEqual(files["new.txt"], b'hello123')
+        r = MyInst().read_resource('new.txt', filesystem='my-s3')
+        self.assertEqual(r, b'hello123')
